@@ -137,18 +137,16 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         self._pool_election.start()
 
         if CONF['service:pool_manager'].enable_recovery_timer:
-            LOG.info(_LI('Starting periodic recovery timer'))
-            self.tg.add_timer(
-                CONF['service:pool_manager'].periodic_recovery_interval,
-                self.periodic_recovery,
-                CONF['service:pool_manager'].periodic_recovery_interval)
+            interval = CONF['service:pool_manager'].periodic_recovery_interval
+            LOG.info(_LI('Starting periodic recovery timer every'
+                         ' %(interval)s s') % {'interval': interval})
+            self.tg.add_timer(interval, self.periodic_recovery, interval)
 
         if CONF['service:pool_manager'].enable_sync_timer:
-            LOG.info(_LI('Starting periodic synchronization timer'))
-            self.tg.add_timer(
-                CONF['service:pool_manager'].periodic_sync_interval,
-                self.periodic_sync,
-                CONF['service:pool_manager'].periodic_sync_interval)
+            interval = CONF['service:pool_manager'].periodic_sync_interval
+            LOG.info(_LI('Starting periodic synchronization timer every'
+                         ' %(interval)s s') % {'interval': interval})
+            self.tg.add_timer(interval, self.periodic_sync, interval)
 
     def stop(self):
         self._pool_election.stop()
@@ -169,72 +167,81 @@ class Service(service.RPCService, coordination.CoordinationMixin,
     # Periodioc Tasks
     def periodic_recovery(self):
         """
+        Runs only on the pool leader
         :return: None
         """
         # NOTE(kiall): Only run this periodic task on the pool leader
-        if self._pool_election.is_leader:
-            context = DesignateContext.get_admin_context(all_tenants=True)
+        if not self._pool_election.is_leader:
+            return
 
-            LOG.debug("Starting Periodic Recovery")
+        context = DesignateContext.get_admin_context(all_tenants=True)
+        LOG.debug("Starting Periodic Recovery")
 
-            try:
-                # Handle Deletion Failures
-                domains = self._get_failed_domains(context, DELETE_ACTION)
+        try:
+            # Handle Deletion Failures
+            domains = self._get_failed_domains(context, DELETE_ACTION)
 
-                for domain in domains:
-                    self.delete_domain(context, domain)
+            for domain in domains:
+                self.delete_domain(context, domain)
 
-                # Handle Creation Failures
-                domains = self._get_failed_domains(context, CREATE_ACTION)
+            # Handle Creation Failures
+            domains = self._get_failed_domains(context, CREATE_ACTION)
 
-                for domain in domains:
-                    self.create_domain(context, domain)
+            for domain in domains:
+                self.create_domain(context, domain)
 
-                # Handle Update Failures
-                domains = self._get_failed_domains(context, UPDATE_ACTION)
+            # Handle Update Failures
+            domains = self._get_failed_domains(context, UPDATE_ACTION)
 
-                for domain in domains:
-                    self.update_domain(context, domain)
+            for domain in domains:
+                self.update_domain(context, domain)
 
-            except Exception:
-                LOG.exception(_LE('An unhandled exception in periodic '
-                                  'recovery occurred'))
+        except Exception:
+            LOG.exception(_LE('An unhandled exception in periodic '
+                              'recovery occurred'))
 
     def periodic_sync(self):
-        """
+        """Periodically sync all the zones that are not in ERROR status
+        Runs only on the pool leader
         :return: None
         """
-        # NOTE(kiall): Only run this periodic task on the pool leader
-        if self._pool_election.is_leader:
-            context = DesignateContext.get_admin_context(all_tenants=True)
+        if not self._pool_election.is_leader:
+            return
 
-            LOG.debug("Starting Periodic Synchronization")
+        context = DesignateContext.get_admin_context(all_tenants=True)
 
-            criterion = {
-                'pool_id': CONF['service:pool_manager'].pool_id,
-                'status': '!%s' % ERROR_STATUS
-            }
+        LOG.debug("Starting Periodic Synchronization")
 
-            periodic_sync_seconds = \
-                CONF['service:pool_manager'].periodic_sync_seconds
+        criterion = {
+            'pool_id': CONF['service:pool_manager'].pool_id,
+            'status': '!%s' % ERROR_STATUS
+        }
 
-            if periodic_sync_seconds is not None:
-                # Generate the current serial, will provide a UTC Unix TS.
-                current = utils.increment_serial()
-                criterion['serial'] = ">%s" % (current - periodic_sync_seconds)
+        periodic_sync_seconds = \
+            CONF['service:pool_manager'].periodic_sync_seconds
 
-            domains = self.central_api.find_domains(context, criterion)
+        if periodic_sync_seconds is not None:
+            # Generate the current serial, will provide a UTC Unix TS.
+            current = utils.increment_serial()
+            criterion['serial'] = ">%s" % (current - periodic_sync_seconds)
 
-            try:
-                for domain in domains:
-                    # TODO(kiall): If the domain was created within the last
-                    #              periodic_sync_seconds, attempt to recreate
-                    #              to fill in targets which may have failed.
-                    self.update_domain(context, domain)
+        domains = self.central_api.find_domains(context, criterion)
 
-            except Exception:
-                LOG.exception(_LE('An unhandled exception in periodic '
-                                  'synchronization occurred.'))
+        try:
+            for domain in domains:
+                # TODO(kiall): If the zone was created within the last
+                #              periodic_sync_seconds, attempt to recreate
+                #              to fill in targets which may have failed.
+                success = self.update_domain(context, domain)
+                if not success:
+                    self.central_api.update_status(
+                        context, domain.id, ERROR_STATUS, domain.serial)
+
+        except Exception:
+            LOG.exception(_LE('An unhandled exception in periodic '
+                              'synchronization occurred.'))
+            self.central_api.update_status(context, domain.id, ERROR_STATUS,
+                                           domain.serial)
 
     # Standard Create/Update/Delete Methods
 
@@ -303,32 +310,30 @@ class Service(service.RPCService, coordination.CoordinationMixin,
             return False
 
     def update_domain(self, context, domain):
-        """
+        """Update a domain across every pool target, check for consensus and
+        for propagation.
         :param context: Security context information.
         :param domain: Domain to be updated
-        :return: None
+        :return: consensus reached (bool)
         """
         LOG.info(_LI("Updating domain %s"), domain.name)
 
-        results = []
-
         # Update the domain on each of the Pool Targets
+        success_count = 0
         for target in self.pool.targets:
-            results.append(
-                self._update_domain_on_target(context, target, domain))
+            ok_status = self._update_domain_on_target(context, target, domain)
+            if ok_status:
+                success_count += 1
 
-        if self._exceed_or_meet_threshold(results.count(True)):
-            LOG.debug('Consensus reached for updating domain %(domain)s '
-                      'on pool targets' % {'domain': domain.name})
+        if not self._exceed_or_meet_threshold(success_count):
+            LOG.warn(_LW('Consensus not reached for updating zone %(zone)s'
+                         ' on pool targets') % {'zone': domain.name})
+            self.central_api.update_status(context, domain.id, ERROR_STATUS,
+                                           domain.serial)
+            return False
 
-        else:
-            LOG.warn(_LW('Consensus not reached for updating domain %(domain)s'
-                         ' on pool targets') % {'domain': domain.name})
-
-            self.central_api.update_status(
-                    context, domain.id, ERROR_STATUS, domain.serial)
-
-            return
+        LOG.debug('Consensus reached for updating zone %(zone)s '
+                  'on pool targets' % {'zone': domain.name})
 
         # Send a NOTIFY to each also-notifies
         for also_notify in self.pool.also_notifies:
@@ -349,8 +354,10 @@ class Service(service.RPCService, coordination.CoordinationMixin,
                 context, domain, nameserver, self.timeout,
                 self.retry_interval, self.max_retries, self.delay)
 
+        return True
+
     def _update_domain_on_target(self, context, target, domain):
-        """
+        """Instruct the appropriate backend to update a zone on a target
         :param context: Security context information.
         :param target: Target to update Domain on
         :param domain: Domain to be updated
@@ -499,6 +506,12 @@ class Service(service.RPCService, coordination.CoordinationMixin,
                     self.central_api.update_status(
                         context, domain.id, ERROR_STATUS, error_serial)
 
+            if status == NO_DOMAIN_STATUS and action != DELETE_ACTION:
+                LOG.warn(_LW('Domain %(domain)s is not present in some '
+                             'targets') % {'domain': domain.name})
+                self.central_api.update_status(
+                    context, domain.id, NO_DOMAIN_STATUS, 0)
+
             if consensus_serial == domain.serial and self._is_consensus(
                     context, domain, action, SUCCESS_STATUS,
                     MAXIMUM_THRESHOLD):
@@ -509,7 +522,7 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         criterion = {
             'pool_id': CONF['service:pool_manager'].pool_id,
             'action': action,
-            'status': 'ERROR'
+            'status': ERROR_STATUS
         }
         return self.central_api.find_domains(context, criterion)
 
@@ -522,10 +535,12 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         return (Decimal(count) / Decimal(total_count)) * Decimal(100)
 
     def _exceed_or_meet_threshold(self, count, threshold=None):
+        """Evaluate if count / the number of pool targets >= threshold
+        Used to implement consensus
+        """
         threshold = threshold or self.threshold
-
-        return self._percentage(
-            count, len(self.pool.targets)) >= Decimal(threshold)
+        perc = self._percentage(count, len(self.pool.targets))
+        return perc >= Decimal(threshold)
 
     @staticmethod
     def _get_sorted_serials(pool_manager_statuses, descending=False):
@@ -542,6 +557,10 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         return self._get_sorted_serials(pool_manager_statuses, descending=True)
 
     def _is_consensus(self, context, domain, action, status, threshold=None):
+        """Fetch zone status across all nameservers through MiniDNS and compare
+        it with the expected `status`
+        :return: consensus reached (bool)
+        """
         status_count = 0
         pool_manager_statuses = self._retrieve_statuses(
             context, domain, action)
@@ -583,6 +602,9 @@ class Service(service.RPCService, coordination.CoordinationMixin,
     # value the nameserver
     @staticmethod
     def _build_status_object(nameserver, domain, action):
+        """
+        :return: :class:`objects.PoolManagerStatus`
+        """
         values = {
             'nameserver_id': nameserver.id,
             'domain_id': domain.id,
@@ -617,6 +639,10 @@ class Service(service.RPCService, coordination.CoordinationMixin,
                 pass
 
     def _retrieve_from_mdns(self, context, nameserver, domain, action):
+        """Instruct MiniDNS to get a zone serial number from a nameserver
+        Set error status if the zone is unexpectedly missing.
+        :return: :class:`objects.PoolManagerStatus` or None
+        """
         try:
             (status, actual_serial, retries) = \
                 self.mdns_api.get_serial_number(
@@ -635,16 +661,16 @@ class Service(service.RPCService, coordination.CoordinationMixin,
 
         if status == NO_DOMAIN_STATUS:
             if action == CREATE_ACTION:
-                pool_manager_status.status = 'ERROR'
+                pool_manager_status.status = ERROR_STATUS
             elif action == DELETE_ACTION:
-                pool_manager_status.status = 'SUCCESS'
-            # TODO(Ron): Handle this case properly.
+                pool_manager_status.status = SUCCESS_STATUS
             elif action == UPDATE_ACTION:
-                pool_manager_status.status = 'ERROR'
+                pool_manager_status.action = CREATE_ACTION
+                pool_manager_status.status = ERROR_STATUS
         else:
             pool_manager_status.status = status
-        pool_manager_status.serial_number = actual_serial \
-            if actual_serial is not None else 0
+
+        pool_manager_status.serial_number = actual_serial or 0
         LOG.debug('Retrieved status %s and serial %s for domain %s '
                   'on nameserver %s with action %s from mdns.' %
                   (pool_manager_status.status,
@@ -655,6 +681,11 @@ class Service(service.RPCService, coordination.CoordinationMixin,
         return pool_manager_status
 
     def _retrieve_statuses(self, context, domain, action):
+        """Instruct MiniDNS to get a zone serial number from all nameservers,
+        unless a cached value is available.
+        Set error status if the zone is unexpectedly missing.
+        :return: list of :class:`objects.PoolManagerStatus`
+        """
         pool_manager_statuses = []
         for nameserver in self.pool.nameservers:
             try:
